"""Provides Additional Extensions for Snowpark"""


from .dataframe_extensions import *
from .functions_extensions import *
from .session_builder_extensions import *
from .types_extensions import *
from .column_extensions import *
from .utils import display


def get_display_html() -> None:
    import inspect
    for frame in inspect.getouterframes(inspect.currentframe()):
        global_names = set(frame.frame.f_globals)
        # Use multiple functions to reduce risk of mismatch
        if all(v in global_names for v in ["displayHTML", "display", "spark"]):
            return frame.frame.f_globals["displayHTML"]
    raise Exception("Unable to detect displayHTML function")

def load_ipython_extension(ipython):
    def instructions():
        inst = """
import pandas as pd
from snowflake.snowpark import Session
import snowpark_extensions
from snowflake.snowpark.functions import col, lit
from snowflake.snowpark import functions as F
        """
        return inst 
    error_message_template="""<div style="background-color: #f44336; color: white; padding: 16px;"><strong>Error:</strong> <span id="error-message">@error</span></div>"""
    output_cell_output = None
    try:
        output_cell_output = get_display_html()
        print("displayHTML detected")
    except:
        from IPython.display import display, HTML
        output_cell_output = lambda x: display(HTML(x))
    from IPython.core.magic import (Magics, magics_class, cell_magic)
    @magics_class
    class SnowparkMagics(Magics):
        def __init__(self, shell):
            super(SnowparkMagics, self).__init__(shell)
        @cell_magic
        def sql(self, line, cell):
            if "session" in self.shell.user_ns:
                session = self.shell.user_ns['session']
                from jinja2 import Template
                t = Template(cell)
                res = t.render(self.shell.user_ns)
                name = None
                if line and line.strip():
                    name = line.strip().split(" ")[0]
                from snowflake.snowpark.exceptions import SnowparkSQLException
                try:
                    df = session.sql(res)
                    html = df.to_pandas().to_html()
                    output_cell_output(html)
                    if name:
                        self.shell.user_ns[name] = df
                    else:
                        self.shell.user_ns["__df"] = df
                except SnowparkSQLException as sce:
                    error_msg = sce.message
                    formatted = error_message_template.replace("@error", error_msg)
                    output_cell_output(formatted)
                except Exception as ex:
                    error_message = str(ex)
                    output_cell_output(f"<pre>{error_message}</pre>")                    
            else:
                return "No session was found. You can setup one by running: session = Session.builder.from_env().getOrCreate()"
    magics = SnowparkMagics(ipython)
    ipython.register_magics(magics)
    compiled_file = "prep_log.py"
    code = compile(instructions(), compiled_file, 'exec')
    async def main():
        await ipython.run_code(code,{})
    import nest_asyncio
    nest_asyncio.apply()
    import asyncio
    asyncio.get_event_loop().run_until_complete(main())




import logging
import configparser
from pathlib import Path
from snowflake.snowpark import Session
import os
import shortuuid
import io

if not hasattr(Session.SessionBuilder,"___extended"):         

    _logger = logging.getLogger(__name__)

    def console_handler(stream='stdout'):
        """
        Create a handler for logging to the original console.
        """
        assert stream in {'stdout', 'stderr'}, "stream must be one of 'stdin' or 'stdout'"
        try:
            # Get the file handle of the original std stream.
            fh = getattr(sys, stream)._original_stdstream_copy
            if fh:
                # Create a writable IO stream.
                stream = io.TextIOWrapper(io.FileIO(fh, 'w'))
                # Set up a stream handler.
                return logging.StreamHandler(stream)
        except:
            return None
    _console_handler = console_handler()

    # useful when running from notebooks
    if _console_handler:
        _logger.addHandler(_console_handler)

    Session.SessionBuilder.___extended = True
    Session.SessionBuilder.___create = Session.SessionBuilder.create
    def SessionBuilder_extendedcreate(self):
        session = self.___create()
        if hasattr(self,"__appname__"):
            setattr(session, "__appname__", self.__appname__)
            uuid = shortuuid.uuid()
            session.query_tag = f"APPNAME={session.__appname__};execution_id={uuid}"
        return session
    Session.SessionBuilder.create = SessionBuilder_extendedcreate
    def SessionBuilder_appName(self,name):
        self.__appname__ = name
        return self

    Session.SessionBuilder.appName = SessionBuilder_appName
    def append_tag(self,tag:str):
         session.query_tag = session.query_tag + ";" + tag
    Session.append_tag = append_tag    

    def SessionBuilder_getOrCreate(self):
        from snowflake.snowpark import context
        from snowflake.snowpark.session import _session_management_lock, _active_sessions
        with _session_management_lock:
            if len(_active_sessions) == 1:
                return next(iter(_active_sessions))
        return self.create()

    Session.SessionBuilder.getOrCreate = SessionBuilder_getOrCreate


    def SessionBuilder_env(self):
        sf_user      = os.getenv("SNOW_USER")     or os.getenv("SNOWSQL_USER")
        sf_password  = os.getenv("SNOW_PASSWORD") or os.getenv("SNOWSQL_PWD")
        sf_account   = os.getenv("SNOW_ACCOUNT")  or os.getenv("SNOWSQL_ACCOUNT")
        sf_role      = os.getenv("SNOW_ROLE")     or os.getenv("SNOWSQL_ROLE")
        sf_warehouse = os.getenv("SNOW_WAREHOUSE") or os.getenv("SNOWSQL_WAREHOUSE")
        sf_database  = os.getenv("SNOW_DATABASE") or os.getenv("SNOWSQL_DATABASE")
        if sf_user is not None:
            self._options["user"]     = os.getenv("SNOW_USER") or os.getenv("SNOWSQL_USER")
        if sf_password is not None:
            self._options["password"] = sf_password
        if sf_account is not None:
            self._options["account"]  = sf_account
        if sf_role is not None:
            self._options["role"]     = sf_role
        if sf_warehouse is not None:
            self._options["warehouse"]= sf_warehouse
        if sf_database is not None:
            self._options["database"] = sf_database
        return self

    def SessionBuilder_snowsql_config(self,section=None,configpath=f'{Path.home()}/.snowsql/config'):
        if not os.path.exists(configpath):
            _logger.error(f"No snowsql config found at:{configpath}")
            return self
        config = configparser.ConfigParser()
        config.read(configpath)
        config_section_name = "connections" if section is None else f"connections.{section}"
        if config_section_name in config:
            section = config[config_section_name]
            if section:
                connection_parameters = {
                    "user":section.get("username"),
                    "password":section.get("password"),
                    "account":section.get('accountname'),
                    "role":section.get('rolename'),
                    "warehouse":section.get('warehousename'),
                    "database":section.get("dbname"),
                    "schema":section.get("schemaname")
                }
                self.configs(connection_parameters)
        else:
            _logger.error(f"Config section {config_section_name} not found in snowsql file: {configpath}")
        return self

    Session.SessionBuilder.from_snowsql = SessionBuilder_snowsql_config
    Session.SessionBuilder.from_env = SessionBuilder_env
    Session.SessionBuilder.env = SessionBuilder_env